@@ -15,11 +15,12 @@ from prompt_toolkit.contrib.completers import PathCompleter, WordCompleter, Syst
 from prompt_toolkit.contrib.regular_languages.compiler import compile
 from prompt_toolkit.contrib.regular_languages.completion import GrammarCompleter
 from prompt_toolkit.contrib.regular_languages.lexer import GrammarLexer
-from prompt_toolkit.shortcuts import create_eventloop
 from prompt_toolkit.document import Document
+from prompt_toolkit.interface import CommandLineInterface
 from prompt_toolkit.layout.controls import TokenListControl
+from prompt_toolkit.shortcuts import create_eventloop
 
-from ptpython.python_input import PythonCommandLineInterface, PythonValidator, PythonCompleter
+from ptpython.python_input import PythonInput, PythonValidator, PythonCompleter
 
 from IPython.terminal.embed import InteractiveShellEmbed as _InteractiveShellEmbed
 from IPython.terminal.ipapp import load_default_config
@@ -41,7 +42,7 @@ class IPythonPrompt(TokenListControl):
     def __init__(self, prompt_manager):
         def get_tokens(cli):
             text = prompt_manager.render('in', color=False, just=False)
-            return [(Token.Layout.Prompt, text)]
+            return [(Token.In, text)]
 
         super(IPythonPrompt, self).__init__(get_tokens)
 
@@ -64,7 +65,8 @@ def create_ipython_grammar():
         \s*
         (
             (?P<percent>%)(
-                (?P<magic>pycat|run|loadpy|load)  \s+ (?P<filename>[^\s]+)     |
+                (?P<magic>pycat|run|loadpy|load)  \s+ (?P<py_filename>[^\s]+)  |
+                (?P<magic>cat)                    \s+ (?P<filename>[^\s]+)     |
                 (?P<magic>pushd|cd|ls)            \s+ (?P<directory>[^\s]+)    |
                 (?P<magic>pdb)                    \s+ (?P<pdb_arg>[^\s]+)      |
                 (?P<magic>autocall)               \s+ (?P<autocall_arg>[^\s]+) |
@@ -90,7 +92,8 @@ def create_completer(get_globals, get_locals, magics_manager, alias_manager):
         'alias_name': AliasCompleter(alias_manager),
         'pdb_arg': WordCompleter(['on', 'off'], ignore_case=True),
         'autocall_arg': WordCompleter(['0', '1', '2'], ignore_case=True),
-        'filename': PathCompleter(only_directories=False, file_filter=lambda name: name.endswith('.py')),
+        'py_filename': PathCompleter(only_directories=False, file_filter=lambda name: name.endswith('.py')),
+        'filename': PathCompleter(only_directories=False),
         'directory': PathCompleter(only_directories=True),
         'system': SystemCompleter(),
     })
@@ -139,7 +142,7 @@ class AliasCompleter(Completer):
                                  display_meta=cmd)
 
 
-class IPythonCommandLineInterface(PythonCommandLineInterface):
+class IPythonInput(PythonInput):
     """
     Override our `PythonCommandLineInterface` to add IPython specific stuff.
     """
@@ -151,7 +154,7 @@ class IPythonCommandLineInterface(PythonCommandLineInterface):
         kw['_validator'] = IPythonValidator()
         kw['_python_prompt_control'] = IPythonPrompt(ipython_shell.prompt_manager)
 
-        super(IPythonCommandLineInterface, self).__init__(*a, **kw)
+        super(IPythonInput, self).__init__(*a, **kw)
         self.ipython_shell = ipython_shell
 
 
@@ -159,10 +162,13 @@ class InteractiveShellEmbed(_InteractiveShellEmbed):
     """
     Override the `InteractiveShellEmbed` from IPython, to replace the front-end
     with our input shell.
+
+    :param configure: Callable for configuring the repl.
     """
     def __init__(self, *a, **kw):
         vi_mode = kw.pop('vi_mode', False)
         history_filename = kw.pop('history_filename', None)
+        configure = kw.pop('configure', None)
 
         super(InteractiveShellEmbed, self).__init__(*a, **kw)
 
@@ -170,16 +176,22 @@ class InteractiveShellEmbed(_InteractiveShellEmbed):
             return self.user_ns
 
         self._eventloop = create_eventloop()
-        self._cli = IPythonCommandLineInterface(
+        ipython_input = IPythonInput(
             self,
-            eventloop=self._eventloop,
             get_globals=get_globals, vi_mode=vi_mode,
             history_filename=history_filename)
 
+        if configure:
+            configure(ipython_input)
+
+        self._cli = CommandLineInterface(
+                application=ipython_input.create_application(),
+                eventloop=self._eventloop)
+
     def raw_input(self, prompt=''):
         print('')
         try:
-            string = self._cli.cli.read_input().text
+            string = self._cli.run().text
 
             # In case of multiline input, make sure to append a newline to the input,
             # otherwise, IPython will ask again for more input in some cases.
